(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs/Subject'), require('rxjs/operator/share')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', 'rxjs/Subject', 'rxjs/operator/share'], factory) :
	(factory((global.core = {}),global.ng.core,global.Rx,global.Rx.Observable.prototype));
}(this, (function (exports,_angular_core,rxjs_Subject,rxjs_operator_share) { 'use strict';

var LoadingBarService = (function () {
    function LoadingBarService() {
        this.pending = new rxjs_Subject.Subject();
        this._pendingRequests = 0;
    }
    /**
     * @param {?=} source$
     * @return {?}
     */
    LoadingBarService.prototype.startLoading = function (source$) {
        if (source$) {
            rxjs_operator_share.share.call(source$).subscribe(this.getLoadingObserver());
        }
        this.pending.next({
            started: this._pendingRequests === 0,
            pendingRequests: ++this._pendingRequests,
        });
    };
    /**
     * @return {?}
     */
    LoadingBarService.prototype.endLoading = function () {
        this.pending.next({
            completed: this._pendingRequests === 1,
            pendingRequests: --this._pendingRequests,
        });
    };
    /**
     * @return {?}
     */
    LoadingBarService.prototype.getLoadingObserver = function () {
        var _this = this;
        return {
            next: function (x) { return x; },
            error: function (err) { return _this.endLoading(); },
            complete: function () { return _this.endLoading(); },
        };
    };
    return LoadingBarService;
}());
LoadingBarService.decorators = [
    { type: _angular_core.Injectable },
];
/**
 * @nocollapse
 */
LoadingBarService.ctorParameters = function () { return []; };
var LoadingBarComponent = (function () {
    /**
     * @param {?} _renderer
     * @param {?} loadingBarService
     */
    function LoadingBarComponent(_renderer, loadingBarService) {
        var _this = this;
        this._renderer = _renderer;
        this.includeSpinner = true;
        this._autoIncrement = true;
        this._includeBar = true;
        this._latencyThreshold = 10;
        this._startSize = 0.02;
        this._started = false;
        this._status = 0;
        loadingBarService.pending.subscribe(function (progress) {
            if (progress.started)
                _this.start();
            if (progress.completed)
                _this.complete();
        });
    }
    /**
     * @return {?}
     */
    LoadingBarComponent.prototype.ngAfterViewInit = function () {
        this.hide(this._loadingBarContainer);
        this.hide(this._spinner);
    };
    /**
     * Inserts the loading bar element into the dom, and sets it to 2%
     * @return {?}
     */
    LoadingBarComponent.prototype.start = function () {
        var _this = this;
        this._startTimeout = setTimeout(function () {
            clearTimeout(_this._completeTimeout);
            // do not continually broadcast the started event:
            if (_this._started) {
                return;
            }
            _this._started = true;
            _this._status = 0;
            if (_this._includeBar) {
                _this.show(_this._loadingBarContainer);
            }
            if (_this.includeSpinner) {
                _this.show(_this._spinner);
            }
            _this.set(_this._startSize);
        }, this._latencyThreshold);
    };
    /**
     * Set the loading bar's width to a certain percent.
     *
     * @param {?} n any value between 0 and 1
     * @return {?}
     */
    LoadingBarComponent.prototype.set = function (n) {
        var _this = this;
        if (!this._started) {
            return;
        }
        var /** @type {?} */ pct = (n * 100) + '%';
        this.setElementStyle(this._loadingBar, 'width', pct);
        this._status = n;
        // increment loadingbar to give the illusion that there is always
        // progress but make sure to cancel the previous timeouts so we don't
        // have multiple incs running at the same time.
        if (this._autoIncrement) {
            clearTimeout(this._incTimeout);
            this._incTimeout = setTimeout(function () {
                _this.inc();
            }, 250);
        }
    };
    /**
     * @return {?}
     */
    LoadingBarComponent.prototype.complete = function () {
        var _this = this;
        setTimeout(function () {
            _this.set(1);
            clearTimeout(_this._completeTimeout);
            clearTimeout(_this._startTimeout);
            // Attempt to aggregate any start/complete calls within 500ms:
            _this._completeTimeout = setTimeout(function () {
                _this._started = false;
                _this.hide(_this._loadingBarContainer);
                _this.hide(_this._spinner);
            }, 500);
        }, this._latencyThreshold);
    };
    /**
     * Increments the loading bar by a random amount
     * but slows down as it progresses
     * @return {?}
     */
    LoadingBarComponent.prototype.inc = function () {
        if (this._status >= 1) {
            return;
        }
        var /** @type {?} */ rnd = 0;
        var /** @type {?} */ stat = this._status;
        if (stat >= 0 && stat < 0.25) {
            // Start out between 3 - 6% increments
            rnd = (Math.random() * (5 - 3 + 1) + 3) / 100;
        }
        else if (stat >= 0.25 && stat < 0.65) {
            // increment between 0 - 3%
            rnd = (Math.random() * 3) / 100;
        }
        else if (stat >= 0.65 && stat < 0.9) {
            // increment between 0 - 2%
            rnd = (Math.random() * 2) / 100;
        }
        else if (stat >= 0.9 && stat < 0.99) {
            // finally, increment it .5 %
            rnd = 0.005;
        }
        else {
            // after 99%, don't increment:
            rnd = 0;
        }
        var /** @type {?} */ pct = this._status + rnd;
        this.set(pct);
    };
    /**
     * @param {?} el
     * @return {?}
     */
    LoadingBarComponent.prototype.show = function (el) {
        this.setElementStyle(el, 'display', 'block');
    };
    /**
     * @param {?} el
     * @return {?}
     */
    LoadingBarComponent.prototype.hide = function (el) {
        this.setElementStyle(el, 'display', 'none');
    };
    /**
     * @param {?} el
     * @param {?} styleName
     * @param {?} styleValue
     * @return {?}
     */
    LoadingBarComponent.prototype.setElementStyle = function (el, styleName, styleValue) {
        this._renderer.setStyle(el.nativeElement, styleName, styleValue);
    };
    return LoadingBarComponent;
}());
LoadingBarComponent.decorators = [
    { type: _angular_core.Component, args: [{
                selector: 'ng-loading-bar',
                template: "\n        <div id=\"loading-bar-spinner\" #loadingBarSpinner><div class=\"spinner-icon\"></div></div>\n        <div id=\"loading-bar\" #loadingBarContainer><div class=\"bar\" #loadingBar><div class=\"peg\"></div></div></div>\n    ",
            },] },
];
/**
 * @nocollapse
 */
LoadingBarComponent.ctorParameters = function () { return [
    { type: _angular_core.Renderer2, },
    { type: LoadingBarService, },
]; };
LoadingBarComponent.propDecorators = {
    '_spinner': [{ type: _angular_core.ViewChild, args: ['loadingBarSpinner',] },],
    '_loadingBarContainer': [{ type: _angular_core.ViewChild, args: ['loadingBarContainer',] },],
    '_loadingBar': [{ type: _angular_core.ViewChild, args: ['loadingBar',] },],
    'includeSpinner': [{ type: _angular_core.Input },],
};
var LoadingBarModule = (function () {
    function LoadingBarModule() {
    }
    return LoadingBarModule;
}());
LoadingBarModule.decorators = [
    { type: _angular_core.NgModule, args: [{
                declarations: [LoadingBarComponent],
                exports: [LoadingBarComponent],
                providers: [LoadingBarService],
            },] },
];
/**
 * @nocollapse
 */
LoadingBarModule.ctorParameters = function () { return []; };

exports.LoadingBarModule = LoadingBarModule;
exports.LoadingBarService = LoadingBarService;
exports.LoadingBarComponent = LoadingBarComponent;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=core.umd.js.map
